var HTML5DemoTableNoScrollRangeElementWrapper;

(function ()
{
	/* This HTML5 demo table control is used to demonstrate how an IEC array can be displayed and used in a HTML5 control.
	 * The complete array is transfered when a value in the array has change. A cell in the table can be edited and the change can be transferred to IEC.  */
	HTML5DemoTableNoScrollRangeElementWrapper = function()
	{
        this.domNode = document.createElement("div");
        this.parentNode = document.createElement("table");
		this.table = this.parentNode;
        this.domNode.appendChild(this.parentNode);
		this.domNode.style.width = "100%";
		this.domNode.style.height = "100%";
		this.domNode.style.overflow = "auto";
		document.body.appendChild(this.domNode);
		
		var caption = this.parentNode.createCaption();
		caption.innerHTML = 'HTML5 Demo Table';

		this._nestedArrays = null;
		this._structuredInfo = null;
		this._startIndex = 0; // startIndex is the first valid index in the array
		this._endIndex = 200; // maxIndex is the last valid index in the array
		
		this._lastScrollPosition = 0;
		this._visibleRows = 0;
		this._rowHeight = 34;
		
	};
		
	HTML5DemoTableNoScrollRangeElementWrapper.prototype =
	{
		_retrieveArrayInfo: function(typeid)
		{
			var self = this;
			var resultPromise = window.CDSWebVisuAccess.getTypeDesc(typeid);
			return resultPromise.then(function(value)  
			{
				if (value.Dimensions !== undefined)
				{
					if (value.Dimensions.length === 2)
					{
						self._startIndex = value.Dimensions[0].MinRange;
						self._endIndex = value.Dimensions[0].MaxRange;
						return true;
					}
					if (value.Dimensions.length === 1)
					{
						self._startIndex = value.Dimensions[0].MinRange;
						self._endIndex = value.Dimensions[0].MaxRange;
						return window.CDSWebVisuAccess.getTypeDesc(value.BaseTypeId);
					}
				}

				return false;
			}).then(function(innerInfo)
			{
				if (typeof innerInfo === 'boolean')
					self._nestedArrays = innerInfo;
				else 
				{
					if (innerInfo.Dimensions !== undefined)
					{
						self._startIndex = innerInfo.Dimensions[0].MinRange;
						self._nestedArrays = innerInfo.Dimensions !== undefined;
					}
					else
					{
						// Array of struct
						self._startIndex = 0;
						self._nestedArrays = false;
						self._structuredInfo = innerInfo;
					}
				}
				return null;		// return value not relevant, just for further chaining
			}).catch(function()
			{
				self._nestedArrays = false;
			});
		},
		
		_getValueToSend: function(iIndex1, newValue)
		{
			var valueToSend;
			if (this._structuredInfo !== undefined)
			{
				if (this._structuredInfo.Components[iIndex1].TypeId === 16 || this._structuredInfo.Components[iIndex1].TypeId === 17) {
					valueToSend = newValue;
				} else if (this._structuredInfo.Components[iIndex1].TypeId === 0) {
					if (newValue === "true" || newValue === "True" || newValue === "TRUE" || newValue === "1")
					{
						valueToSend = 1;
					}
					else if (newValue === "false" || newValue === "False" || newValue === "FALSE" || newValue === "0")
					{
						valueToSend = 0;
					}
				} else {
					valueToSend = Number(newValue);
				}
			}
			else
				valueToSend = Number(newValue);
			return valueToSend;
		},
		
		_sendCellValue: function(cell, oldValue, newValue)
		{
			var iIndex0, iIndex1, iIndex2=0, valueToSend;
			iIndex0 = cell.parentNode.rowIndex;
			iIndex1 = cell.cellIndex-1; // -1 because we have a row header
			valueToSend = this._getValueToSend(iIndex1, newValue);
			
			var self = this;
			var result = window.CDSWebVisuAccess.checkComplexValue('setValue', valueToSend, 2, iIndex0, iIndex1, iIndex2);
			result.then(function(value) 
			{
				self.ValueOk(cell, oldValue, newValue, iIndex0, iIndex1);
			}, function(value) {
				self.ValueNOk(cell, oldValue, newValue, iIndex0, iIndex1);
			});
		},
		
		ValueOk: function(cell, oldValue, newValue, iIndex0, iIndex1)
		{
			var iIndex2=0, valueToSend;
			valueToSend = this._getValueToSend(iIndex1, newValue);
			window.CDSWebVisuAccess.sendComplexValue('setValue', valueToSend, 2, iIndex0, iIndex1, iIndex2);
			console.log('Valid value ', newValue);
		},
		
		ValueNOk: function(cell, oldValue, newValue, iIndex0, iIndex1)
		{
			cell.innerHTML = oldValue;
			console.log('Invalid value ', newValue);
		},
		
		_createCell: function(cell, value)
		{
			cell.innerHTML = value;
			var self = this;
			cell.onclick = function() 
			{
				if (this.hasAttribute('data-clicked')) {
					return;
				}
				this.setAttribute('data-clicked', 'yes');
				this.setAttribute('data-text', this.innerHTML);

				var input = document.createElement('input');
				input.setAttribute('type', 'text');
				input.value = this.innerHTML;
				input.style.width = this.offsetWidth - (this.clientLeft * 2) + "px";
				input.style.height = this.offsetHeight - (this.clientTop * 2) + "px"
				input.style.border = "0px";
				input.style.fontFamily = "inherit";
				input.style.fontSize = "inherit";
				input.style.textAlign = "inherit";
				input.style.backgroundColor = "LightGoldenRodYellow";
				
				input.onblur = function() {
					var td = input.parentElement;
					var orig_text = input.parentElement.getAttribute('data-text');
					var current_text =  this.value;
					
					if (orig_text != current_text) {
						// there are changes in the cell's text
						td.removeAttribute('data-clicked');
						td.removeAttribute('data-text');
						td.innerHTML = current_text;
						td.style.cssText = 'padding: 5px';
						console.log(orig_text + ' is changed to ' + current_text);
						self._sendCellValue(td, orig_text, current_text);
					} else {
						td.removeAttribute('data-clicked');
						td.removeAttribute('data-text');
						td.innerHTML = orig_text;
						td.style.cssText = 'padding: 5px';
					}
				}
				
				input.onkeypress = function() {
					if (event.keyCode == 13) {
						this.blur();
					}
				}

				this.innerHTML = ''
				this.style.cssText = 'padding: 0px 0px';
				this.append(input);
				this.firstElementChild.select();
			}
		},

		_doSetValue : function(value, type, typeid)
		{

			if (this.table.rows.length === 0)
			{
				// Create table an set values
				if (value instanceof Array)
				{
					var myArray = value, i=0;
					for (i = 0; i < myArray.length; i++) 
					{
						var row = this.table.insertRow(-1);
						headerCell = document.createElement("th");
						row.appendChild(headerCell);
						headerCell.innerHTML = this._startIndex + i;
						if (myArray[i] instanceof Array)
						{
							for (var j = 0; j < myArray[i].length; j++) 
							{
								var cell = row.insertCell();
								this._createCell(cell, myArray[i][j]);
							}
						}	
						else
						{
							var cell = row.insertCell();
							this._createCell(cell, myArray[i]);
						}
					}
					
					if (myArray.length <= this._endIndex)
					{
						while (i <= this._endIndex)
						{
							var row = this.table.insertRow(-1);
							headerCell = document.createElement("th");
							row.appendChild(headerCell);
							headerCell.innerHTML = this._startIndex + i;
							if (myArray[0] instanceof Array)
							{
								for (var j = 0; j < myArray[0].length; j++) 
								{
									var cell = row.insertCell();
									this._createCell(cell, '');
								}
							}	
							else
							{
								var cell = row.insertCell();
								this._createCell(cell, '');
							}
							i++;
						}
					}
				}
			}
			else
			{
				// Only update the values
				if (value instanceof Array)
				{
					var myArray = value;
					var rows = this.table.rows;
					if (rows.length >= myArray.length)
					{
						for (var i = this._startIndex; i < (this._startIndex + myArray.length); i++) 
						{
							var i2 = i-this._startIndex;
							if (i < rows.length)
							{
								var row = rows[i], cells = row.cells;
								if (myArray[i2] instanceof Array)
								{
									if (row.childNodes.length === myArray[i2].length + 1)
									{
										for (var j = 0; j < myArray[i2].length; j++) 
										{
											var cell = cells[j + 1];	// +j due to header cell
											cell.innerHTML = myArray[i2][j];
										}
									}
								}	
								else
								{
									var cell = cells[i2];
									cell.innerHTML = myArray[i2];
								}
							}
						}
					}
				}
			}
		},
		
		setValue: function(value, type, typeid)
		{
			if (this._nestedArrays == null)
			{
				var self = this;
				function setClosure()
				{
					self._doSetValue(value, type, typeid);
				};

				this._retrieveArrayInfo(typeid).then(setClosure, setClosure);
			}
			else
			{
				// if we already know the nesting info, we can  proceed directly
				this._doSetValue(value, type, typeid);
			}
		}
	};
}());

